/****************

	bfcompile

	Written by Brian Luft (bluft@execpc.com)

 ****************

	FILENAME:		CompileWindow.cpp
	DESCRIPTION:	Implements the CCompileWindow class
	CLASSES:		CCompileWindow

 ****************/

#include <layout-all.h>
#include <Screen.h>
#include <stdlib.h>
#include <stdio.h>
#include <string.h>
#include <Resources.h>
#include <Entry.h>
#include <Debug.h>
#include <Roster.h>
#include "FilenameWindow.h"
#include "CompileWindow.h"

// Global so that our thread function can pick it up 
CFilenameWindow *gParentWnd;
CCompileWindow *gCompileWnd;

/****************/
CCompileWindow::CCompileWindow( CFilenameWindow *Parent ):
	MWindow( BRect(50, 50, 50+250, 50), "Compiler",	B_TITLED_WINDOW,
		B_NOT_RESIZABLE | B_NOT_ZOOMABLE )
{
	gParentWnd = Parent;
	gCompileWnd = this;
	MView *MainView = new MBorder( M_RAISED_BORDER, 5, "",
			new VGroup
			(
				new MBorder( M_LABELED_BORDER, 7, "Progress",
					new VGroup
					(
						new HGroup
						(
							fX1 = new MStringView( "X " ),
							new MStringView( "Stripping BF Source" ),
							new Space, new Space, new Space, new Space, 0
						),
						new HGroup
						(
							fX2 = new MStringView( "X " ),
							new MStringView( "Generating C++ Sources" ),
							new Space, new Space, new Space, new Space, 0
						),
						new HGroup
						(
							fX3 = new MStringView( "X " ),
							new MStringView( "Compiling" ),
							new Space, new Space, new Space, new Space, new Space, new Space, 0
						),
						new HGroup
						(
							fX4 = new MStringView( "X " ),
							new MStringView( "Stripping Binary" ),
							new Space, new Space, new Space, new Space, 0
						),
						new HGroup
						(
							fX5 = new MStringView( "X " ),
							new MStringView( "Mimesetting Binary" ),
							new Space, new Space, new Space, new Space, 0
						),
						0
					)
				),
				new HGroup
				(
					new Space,
					fClose = new MButton( "Close", new BMessage(kClose) ),
					new Space,
					0
				),
				0
			)
		);
	
	AddChild( dynamic_cast<BView*>(MainView) );
	fClose->MakeDefault( true );
	fClose->SetEnabled( false );
	
	// Make the Xs all bold
	fX1->SetFont( be_bold_font );
	fX2->SetFont( be_bold_font );
	fX3->SetFont( be_bold_font );
	fX4->SetFont( be_bold_font );
	fX5->SetFont( be_bold_font );
	
	// Make them all light gray
	fX1->SetHighColor( 192, 192, 192, 255 );
	fX2->SetHighColor( 192, 192, 192, 255 );
	fX3->SetHighColor( 192, 192, 192, 255 );
	fX4->SetHighColor( 192, 192, 192, 255 );
	fX5->SetHighColor( 192, 192, 192, 255 );
	
	BScreen Screen;

	MoveTo(
		Screen.Frame().Width() / 2 - Frame().Width() / 2,
		100 );
	
	// Start compiling that shiznit
	thread_id Compiler = spawn_thread(
		(thread_func)__Compiler,
		"Compiler",
		B_NORMAL_PRIORITY,
		NULL );
	resume_thread( Compiler );
}

/****************/
CCompileWindow::~CCompileWindow( void )
{
}

/****************/
void CCompileWindow::MessageReceived( BMessage *Message )
{
	switch( Message->what )
	{
		case kClose:
			Quit();
			break;
		
		default:
			MWindow::MessageReceived( Message );
			break;
	}
}

/****************/
void __Compiler( void )
{
	const char *BF = gParentWnd->fInputFilename->Text();
	const char *Output = gParentWnd->fOutputFilename->Text();
	int32 Optimize = gParentWnd->fOptimize->Value();
	int32 LeaveCSrc = gParentWnd->fLeaveCSrc->Value();
	
	// ** First, make a copy of the sources
	char buf[1024];
	sprintf( buf, "cp %s /boot/var/tmp/bf-work", BF );
	system( buf );
	
	// ** Strip the BF sources
	system( "bfstrip /boot/var/tmp/bf-work" );
	gCompileWnd->fX1->LockLooper();
	gCompileWnd->fX1->SetHighColor( 0, 0, 0, 255 );
	gCompileWnd->fX1->Invalidate();
	gCompileWnd->fX1->UnlockLooper();
	
	// ** Generate the C++ sources
	// read bf.cpp from resources
	size_t Size;
	BResources Res;
	entry_ref MyPath;
	app_info Info;
	
	be_app->GetAppInfo( &Info ); // Get the path to this executable
	MyPath = Info.ref;
	BFile MyFile( &MyPath, B_READ_ONLY );
	Res.SetTo( &MyFile );
	char *BFCPP = (char *)Res.LoadResource( B_STRING_TYPE, "bf.cpp", &Size );
	
	// Write bf.cpp to a file
	FILE *CPPFP = fopen( "/boot/var/tmp/bf.cpp", "w" );
	fprintf( CPPFP, "%s", BFCPP );
	fclose( CPPFP );

	// Generate bf.h
	// Find out how big it is
	struct stat BFStat;
	stat( BF, &BFStat );
	FILE *BFFP = fopen( "/boot/var/tmp/bf-work", "r" );
	char BFSrc[ BFStat.st_size+1 ];
	fgets( BFSrc, sizeof( BFSrc ), BFFP );
	fclose( BFFP );
	
	FILE *HFP = fopen( "/boot/var/tmp/bf.h", "w" );
	ASSERT( HFP );
	fprintf( HFP, "char PROG[] = \"%s\";", BFSrc );
	fclose( HFP );
	gCompileWnd->fX2->LockLooper();
	gCompileWnd->fX2->SetHighColor( 0, 0, 0, 255 );
	gCompileWnd->fX2->Invalidate();
	gCompileWnd->fX2->UnlockLooper();
	
	// ** Compile
	if( Optimize )
		system( "gcc -O3 /boot/var/tmp/bf.cpp -o /boot/var/tmp/MyBFApp" );
	else
		system( "gcc -O0 /boot/var/tmp/bf.cpp -o /boot/var/tmp/MyBFApp" );
	gCompileWnd->fX3->LockLooper();
	gCompileWnd->fX3->SetHighColor( 0, 0, 0, 255 );
	gCompileWnd->fX3->Invalidate();
	gCompileWnd->fX3->UnlockLooper();
	
	// ** Strip binary
	system( "strip /boot/var/tmp/MyBFApp" );
	gCompileWnd->fX4->LockLooper();
	gCompileWnd->fX4->SetHighColor( 0, 0, 0, 255 );
	gCompileWnd->fX4->Invalidate();
	gCompileWnd->fX4->UnlockLooper();
	
	// ** Mimeset binary
	system( "mimeset /boot/var/tmp/MyBFApp" );
	gCompileWnd->fX5->LockLooper();
	gCompileWnd->fX5->SetHighColor( 0, 0, 0, 255 );
	gCompileWnd->fX5->Invalidate();
	gCompileWnd->fX5->UnlockLooper();
	
	// Move to the output
	memset( buf, 0, 1024 );
	sprintf( buf, "mv /boot/var/tmp/MyBFApp %s", Output );
	system( buf );
	
	// Clean up
	if( LeaveCSrc )
	{
		system( "cp /boot/var/tmp/bf.cpp /boot/home/bf.cpp" );
		system( "cp /boot/var/tmp/bf.h /boot/home/bf.h" );
	}
	
	system( "rm /boot/var/tmp/bf.cpp" );
	system( "rm /boot/var/tmp/bf.h" );
	system( "rm /boot/var/tmp/bf-work" );
	
	// Done!
	gCompileWnd->fClose->LockLooper();
	gCompileWnd->fClose->SetEnabled( true );
	gCompileWnd->fClose->UnlockLooper();
}