/****************

	bfcompile

	Written by Brian Luft (bluft@execpc.com)

 ****************

	FILENAME:		FilenameWindow.cpp
	DESCRIPTION:	Implements the CFilenameWindow class
	CLASSES:		CFilenameWindow

 ****************/

#include <layout-all.h>
#include <Screen.h>
#include <Alert.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <FilePanel.h>
#include "FilenameWindow.h"
#include "CompileWindow.h"

/****************/
CFilenameWindow::CFilenameWindow( void ):
	MWindow( BRect(50, 50, 50+325, 50), "Compiler Setup", B_TITLED_WINDOW,
		B_NOT_RESIZABLE | B_NOT_ZOOMABLE )
{
	MButton *OKBtn;
	MView *MainView = new MBorder( M_RAISED_BORDER, 5, "",
			new VGroup
			(
				new MBorder( M_LABELED_BORDER, 7, "Files",
					new VGroup
					(
						fInputFilename = new MTextControl( "BF Source Input:", "" ),
						new HGroup
						(
							new Space, new Space, new Space,
							new MButton( kBrowse, new BMessage(kBrowseInput) ), 0
						),
						fOutputFilename = new MTextControl( "Binary Output:", "" ),
						new HGroup
						(
							new Space, new Space, new Space, 
							new MButton( kBrowse, new BMessage(kBrowseOutput) ), 0
						),
						0
					)
				),
				new MBorder( M_LABELED_BORDER, 7, "Flags",
					new VGroup
					(
						fOptimize = new MCheckBox( "Optimize" ),
						fLeaveCSrc = new MCheckBox( "Leave C Sources" ),
						0
					)
				),
				new HGroup
				(
					new MButton( "About", new BMessage(kAbout) ),
					new MButton( "Cancel", new BMessage(kCancel) ),
					OKBtn = new MButton( "OK", new BMessage(kOK) ),
					0
				),
				0
			)
		);
	
	AddChild( dynamic_cast<BView*>(MainView) );
	OKBtn->MakeDefault( true ); // Make OK the default
	fOptimize->SetValue( 1 ); // Make Optimize checked
	DivideSame( fInputFilename, fOutputFilename, 0 );
	// Center it
	BScreen Screen;

	MoveTo(
		Screen.Frame().Width() / 2 - Frame().Width() / 2,
		100);
}

/****************/
CFilenameWindow::~CFilenameWindow( void )
{
}

/****************/
void CFilenameWindow::MessageReceived( BMessage *Message )
{
	switch( Message->what )
	{
		case kBrowseInput:
		{
			FILE *OpenDlg = popen( "/boot/home/config/bin/filedlg --open", "r" );
			char Filename[1024];
			fgets( Filename, 1024, OpenDlg );
			pclose( OpenDlg );
			// Make sure it really exists
			FILE *Tester = fopen( strtok(Filename, "\n"), "r" );
			if( Tester )
			{
				fInputFilename->SetText( strtok(Filename, "\n") );
				fclose( Tester );
			}
			break;
		}
		
		case kBrowseOutput:
		{
			FILE *SaveDlg = popen( "/boot/home/config/bin/filedlg --save", "r" );
			char Filename[1024];
			fgets( Filename, 1024, SaveDlg );
			pclose( SaveDlg );
			// Make sure it isn't bogus
			if( strstr(Filename, "/") )
				fOutputFilename->SetText( strtok(Filename, "\n") );
			break;
		}

		case kOK:
		{
			// Make sure we have data!
			if( strlen( fInputFilename->Text() ) == 0 ||
				strlen( fOutputFilename->Text() ) == 0 )
			{
				BAlert *NeedFilename = new BAlert(
					"bfcompile",
					"You need to enter a filename into both the Input and the Output textboxes.",
					"OK", NULL, NULL, B_WIDTH_AS_USUAL, B_STOP_ALERT );
				NeedFilename->Go();
				break;
			}
			
			// Make sure the input file exists
			char *Filename = (char*)fInputFilename->Text();
			FILE *Tester = fopen( strtok(Filename, "\n"), "r" );
			if( !Tester )
			{
				BAlert *InvalidFilename = new BAlert(
					"bfcompile",
					"The Input filename is invalid.",
					"OK", NULL, NULL, B_WIDTH_AS_USUAL, B_STOP_ALERT );
				InvalidFilename->Go();
				break;
			}
			
			CCompileWindow *Window = new CCompileWindow( this );
			Window->Show();
			break;
		}
		
		case kCancel:
		{
			be_app->PostMessage( B_QUIT_REQUESTED );
			break;
		}
		
		case kAbout:
		{
			BAlert *AboutAlert = new BAlert(
				"About bfcompile",
				"bfcompile v1.1: a BrainFsck compiler for the BeOS.  Written"
					" by Brian Luft.  Comments, questions, suggestions?"
					" E-mail me at bluft@execpc.com.  Enjoy!",
				"OK", NULL, NULL, B_WIDTH_AS_USUAL, B_IDEA_ALERT );
			AboutAlert->Go();
			break;	
		}
		
		default:
		{
			MWindow::MessageReceived( Message );
			break;
		}
	}
}

/****************/
bool CFilenameWindow::QuitRequested( void )
{
	be_app->PostMessage( B_QUIT_REQUESTED );
	return( true );
}
	